import jwt
import time
import uuid
from django.conf import settings
from django.shortcuts import get_object_or_404
from django.db import models
from django.contrib.auth import get_user_model
from django.utils.translation import gettext_lazy as _
from django.core.exceptions import ValidationError
from django.core.validators import (
    MinLengthValidator,
    MaxLengthValidator,
)

from django_silly_auth.config import SILLY_AUTH_SETTINGS as conf

if conf["VERBOSE"]:
    print("=== DSA IMPORT django_silly_auth.mixins")


class SillyAuthUserMixin(models.Model):
    id = models.UUIDField(
        primary_key=True,
        default=uuid.uuid4,
        editable=False,
    )

    username = models.CharField(
        _("username"),
        max_length=150,
        unique=True,
        help_text=_(
            _("Required. 150 characters or fewer. Letters, digits and ./+/-/_ only."),
        ),
        validators=[
            MinLengthValidator(5),
            MaxLengthValidator(150)
        ],
        error_messages={
            "unique": _("A user with that username already exists."),
            "max_length": _("Username must be less than 150 characters."),
            "invalid": _("Username must contain only letters, digits and ./+/-/_ characters."),
        },
    )
    email = models.EmailField(_("email address"), unique=True)

    new_email = models.EmailField(blank=True, null=True, unique=False)
    is_confirmed = models.BooleanField(default=False)

    class Meta:
        abstract = True

    def get_jwt_token(self, expires_in=conf["EMAIL_VALID_TIME"]):

        token = jwt.encode(
            {'id': str(self.id), 'exp': time.time()+expires_in},
            settings.SECRET_KEY, algorithm='HS256'
        )
        return token

    @staticmethod
    def verify_jwt_token(token):
        try:
            pk = jwt.decode(
                token,
                settings.SECRET_KEY,
                algorithms=['HS256'])['id']
        except Exception as e:
            print("Token error:", e)
            return None
        return get_object_or_404(get_user_model(), id=pk)

    def clean(self):
        if "@" in self.username:
            raise ValidationError(
                {'username': _("A username cannot include the symbol '@'.")})
        super().clean()

    def save(self, *args, **kwargs):
        # self.full_clean()
        super().save(*args, **kwargs)

    def __str__(self):
        return f"<User (DSA): {self.username}>"



from django.contrib.auth import get_user_model
from django.shortcuts import render, redirect
from django.contrib.auth import login, logout, authenticate
from django.contrib import messages
from django.contrib.auth.decorators import login_required
from django.utils.translation import gettext_lazy as _
from django.db import transaction

from django_silly_auth.config import SILLY_AUTH_SETTINGS as conf
from django_silly_auth.forms import (
    LoginForm,
    SignUpForm,
    CredentialForm,
    ResetPasswordForm,
    ChangeUsernameForm,
    ChangeEmailForm,
)
from django_silly_auth.utils import (
    send_password_reset_email,
    send_confirm_email,
    delete_unconfirmed,
)

if conf["VERBOSE"]:
    print("=== DSA IMPORT django_silly_auth.views.classics")

User = get_user_model()


@transaction.atomic
def index(request):
    context = {
        "base_template": conf["BASE_TEMPLATE"],
        "title": conf["TEMPLATES_TITLE"],
    }
    return render(request, conf["CLASSIC_INDEX"], context)

@transaction.atomic
def login_view(request):
    if request.method == 'POST':
        form = LoginForm(request.POST)
        if form.is_valid():
            credential = form.cleaned_data['credential']
            password = form.cleaned_data['password']
            if "@" in credential:
                username = User.objects.get(email=credential).username
                user = authenticate(
                    request, username=username, password=password)
            else:
                user = authenticate(
                    request, username=credential, password=password)
            if user is not None and user.is_confirmed:
                login(request, user)
                return redirect('classic_index')
            else:
                messages.add_message(
                    request, messages.ERROR,
                    message=_("Incorrect credentials or unconfirmed account."),
                    extra_tags="warning"),
        else:
            context = {
                "form": form,
                "base_template": conf["BASE_TEMPLATE"],
                "title": conf["TEMPLATES_TITLE"],
            }
            return render(request, conf["CLASSIC_LOGIN"], context)

    form = LoginForm()
    context = {
        "form": form,
        "base_template": conf["BASE_TEMPLATE"],
        "title": conf["TEMPLATES_TITLE"],
    }
    return render(request, conf["CLASSIC_LOGIN"], context)


def logout_view(request):
    logout(request)
    return redirect('classic_index')


@transaction.atomic
@login_required
def account(request):
    context = {
        "base_template": conf["BASE_TEMPLATE"],
        "title": conf["TEMPLATES_TITLE"],
    }
    return render(request, conf["CLASSIC_ACCOUNT"], context)


@transaction.atomic
def signup_view(request):
    if request.method == 'POST':
        form = SignUpForm(request.POST)
        if form.is_valid():
            username = form.cleaned_data['username']
            email = form.cleaned_data['email']
            password = form.cleaned_data['password']
            user = User(username=username, email=email, is_active=True)
            user.set_password(password)
            user.save()
            send_confirm_email(request, user)
            delete_unconfirmed(user)

            message1 = _(
                "Please check your inbox at"
                " '%(email)s' to confirm your account."
            ) % {'email': user.email}
            message2 = ""
            if conf["DELETE_UNCONFIRMED_TIME"] != 0.0:

                message2 = _(
                    " If you do not confirm your account within the next "
                    "%(time)s hours, it will be deleted."
                ) % {'time': conf['DELETE_UNCONFIRMED_TIME']}

            message = "%(message1)s%(message2)s" % {
                'message1': message1,
                'message2': message2,
            }

            messages.add_message(
                request, messages.INFO,
                message=message,
                extra_tags="info"
            )
            return redirect('classic_login')
        else:
            context = {
                "form": form,
                "base_template": conf["BASE_TEMPLATE"],
                "title": conf["TEMPLATES_TITLE"],
            }
            return render(request, conf["CLASSIC_SIGNUP"], context)

    form = SignUpForm()
    context = {
        "form": form,
        "base_template": conf["BASE_TEMPLATE"],
        "title": conf["TEMPLATES_TITLE"],
    }
    return render(request, conf["CLASSIC_SIGNUP"], context)


@transaction.atomic
def request_password_reset(request):
    if request.method == "POST":
        form = CredentialForm(request.POST)
        if form.is_valid():
            credential = form.cleaned_data['credential']
            if "@" in credential:
                user = User.objects.get(email=credential)
            else:
                user = User.objects.get(username=credential)
            # user existence is checked by the form validation #
            if user.is_active:
                messages.add_message(
                    request, messages.INFO,
                    message=(_(
                        "Please check your inbox "
                        "and follow the instructions "
                        "to reset your password."
                        )),
                    extra_tags="info"
                )
                send_password_reset_email(request, user)

            else:
                messages.add_message(
                    request, messages.INFO,
                    message=(_(
                        "Your account is no longer active. Please contact the administrator. "
                        "No email has been sent."
                        )),
                    extra_tags="info"
                )
        else:
            context = {
                "form": form,
                "base_template": conf["BASE_TEMPLATE"],
                "title": conf["TEMPLATES_TITLE"],
            }
            return render(
                request,
                conf["CLASSIC_REQUEST_PASSWORD_RESET"],
                context
                )
    form = CredentialForm()
    if request.user.is_authenticated:
        form = CredentialForm(initial={'credential': request.user.email})
    context = {
        "form": form,
        "base_template": conf["BASE_TEMPLATE"],
        "title": conf["TEMPLATES_TITLE"],
    }
    return render(request, conf["CLASSIC_REQUEST_PASSWORD_RESET"], context)


@transaction.atomic
def reset_password(request, token=None):
    """Receive the token from the confirmation email and reset the password
    or already authenticated user.
    """
    if token:
        user = User.verify_jwt_token(token)
    elif request.user.is_authenticated:
        user = request.user
    else:
        user = None

    if user is None:
        messages.add_message(
            request, messages.INFO,
            message=(_(
                "Token invalid or expired"
                )),
            extra_tags="danger"
        )
        return redirect('classic_index')

    if request.method == 'POST':
        form = ResetPasswordForm(request.POST)
        if form.is_valid():
            user.set_password(form.cleaned_data['password'])
            user.save()
            if conf["USE_SILLY"]:
                return redirect('dsa_password_reset_done')
            login(request, user)
            messages.add_message(
                request, messages.SUCCESS,
                message=_("Your password has been successfully reset. Please log in."),
                extra_tags="success"
            )
            return redirect('classic_index')
        else:
            context = {
                'form': form,
                "base_template": conf["BASE_TEMPLATE"],
                "title": conf["TEMPLATES_TITLE"],
            }
            return render(request, conf["CLASSIC_RESET_PASSWORD"], context)

    form = ResetPasswordForm()
    login(request, user)
    if token:
        messages.add_message(
            request, messages.SUCCESS,
            message=_("You have been logged in via email confirmation. Please reset your password."),
            extra_tags="warning"
        )

    context = {
        'form': form,
        "base_template": conf["BASE_TEMPLATE"],
        "title": conf["TEMPLATES_TITLE"],
    }
    return render(request, conf["CLASSIC_RESET_PASSWORD"], context)


@transaction.atomic
@login_required
def change_username(request):
    if request.method == 'POST':
        form = ChangeUsernameForm(request.POST)
        if form.is_valid():
            username = form.cleaned_data['username']
            user = request.user
            user.username = username
            user.save()
            messages.add_message(
                request, messages.SUCCESS,
                message=_(
                    "New username set: '%(username)s'."
                ) % {'username': username},
                extra_tags="success"
            )
            return redirect("classic_account")
        else:
            context = {
                'form': form,
                "base_template": conf["BASE_TEMPLATE"],
                "title": conf["TEMPLATES_TITLE"],
            }
            return render(
                request,
                conf["CLASSIC_CHANGE_USERNAME"],
                context,
                )

    form = ChangeUsernameForm()
    context = {
        'form': form,
        "base_template": conf["BASE_TEMPLATE"],
        "title": conf["TEMPLATES_TITLE"],
    }
    return render(request, conf["CLASSIC_CHANGE_USERNAME"], context)


@transaction.atomic
@login_required
def change_email(request):
    if request.method == 'POST':
        form = ChangeEmailForm(request.POST)
        if form.is_valid():
            email = form.cleaned_data['email']
            user = request.user
            user.new_email = email
            user.save()
            send_confirm_email(request, user)

            message = _(
                "Please check your inbox to confirm your new "
                "address at '%(email)s'") % {"email": user.new_email}
            messages.add_message(
                request, messages.INFO,
                message,
                extra_tags="info"
            )
            return redirect("classic_account")
        else:
            context = {
                'form': form,
                "base_template": conf["BASE_TEMPLATE"],
                "title": conf["TEMPLATES_TITLE"],
            }
            return render(request, conf["CLASSIC_CHANGE_EMAIL"], context)

    form = ChangeEmailForm()
    context = {
        'form': form,
        "base_template": conf["BASE_TEMPLATE"],
        "title": conf["TEMPLATES_TITLE"],
    }
    return render(request, conf["CLASSIC_CHANGE_EMAIL"], context)


@transaction.atomic
def confirm_email(request, token):
    user = User.verify_jwt_token(token)
    if user is None:
        messages.add_message(
            request, messages.INFO,
            message=(_(
                "Token invalid or expired"
                )),
            extra_tags="danger"
        )
        return redirect('classic_index')
    if user is not None and user.is_active:
        if not user.is_confirmed:
            user.is_confirmed = True
            user.new_email = None
            msg = _("Your account has been confirmed.")
            user.save()
        elif user.new_email:
            user.email = user.new_email
            user.new_email = None
            msg = _("Your new email has been confirmed.")
            user.save()
        else:
            msg = _("Your account is already confirmed.")
        messages.add_message(
            request, messages.SUCCESS,
            message=msg,
            extra_tags="success"
        )

        return redirect("classic_index")


@transaction.atomic
def request_resend_account_confirmation_email(request):
    if request.method == 'POST':
        form = CredentialForm(request.POST)
        if form.is_valid():
            credential = form.cleaned_data['credential']
            if "@" in credential:
                user = User.objects.get(email=credential)
            else:
                user = User.objects.get(username=credential)

            # user existence is checked by the form validation #

            if user.is_confirmed:
                messages.add_message(
                    request, messages.INFO,
                    message=(_(
                        "Your account is already confirmed. "
                        "No email has been sent."
                        )),
                    extra_tags="info"
                )
                return redirect("classic_login")

            send_confirm_email(request, user)

            messages.add_message(
                request, messages.INFO,
                message=(_(
                    "Please check your inbox "
                    "to confirm your account."
                    )),
                extra_tags="info"
            )
        else:
            context = {
                "form": form,
                "base_template": conf["BASE_TEMPLATE"],
                "title": conf["TEMPLATES_TITLE"],
            }
            return render(
                request,
                conf["CLASSIC_REQUEST_RESEND_ACCOUNT_CONFIRMATION_EMAIL"],
                context
                )
    form = CredentialForm()
    context = {
        "form": form,
        "base_template": conf["BASE_TEMPLATE"],
        "title": conf["TEMPLATES_TITLE"],
    }
    return render(
        request,
        conf["CLASSIC_REQUEST_RESEND_ACCOUNT_CONFIRMATION_EMAIL"],
        context
        )



from django.urls import path
from django.db.models.signals import pre_save
from django.dispatch import receiver
from django.contrib.auth import get_user_model

from django_silly_auth.config import SILLY_AUTH_SETTINGS as conf
from django_silly_auth.views import (
    api_views,
    try_views,
    silly_views,
    classics)


if conf["VERBOSE"]:
    print("=== DSA IMPORT django_silly_auth.urls")
    if conf["USE_DRF"]:
        print("=== DSA LoginWithAuthToken FROM django_silly_auth.views.api_custom_login")

prefix = conf["DSA_PREFIX"]
User = get_user_model()


# Signal interceptor to make sure that superusers are always active
@receiver(pre_save, sender=User)
def new_superuser_is_always_active(sender, instance, **kwargs):
    if instance.is_superuser and not instance.is_confirmed:
        instance.is_confirmed = True
        instance.is_active = True


urlpatterns = []

# DRF routes
if conf["USE_DRF"]:
    urlpatterns += [
        path(f'{prefix}token/login/', api_views.LoginWithAuthToken.as_view(), name="token_login"),
        path(f'{prefix}token/logout/', api_views.token_logout, name="token_logout"),
        path(
            f'{prefix}password/request_reset/',
            api_views.password_request_reset,
            name='password_request_reset'
        ),
        path(
            f'{prefix}email/confirm_email/resend/',
            api_views.email_confirm_email_resend,
            name="email_confirm_email_resend"
        ),
        path(
            f'{prefix}password/change/',
            api_views.password_change,
            name='password_change'
        ),
        path(
            f'{prefix}email/request_change/',
            api_views.email_request_change,
            name='email_request_change'
        ),
        ]

    if conf['ALLOW_CHANGE_USERNAME']:
        urlpatterns += [
            path(
                f'{prefix}username/change/',
                api_views.username_change,
                name='username_change'
            ),
        ]

    if conf["ALLOW_DELETE_ME_ENDPOINT"]:
        urlpatterns += [
            path(f'{prefix}users/delete_me/', api_views.users_delete_me, name='users_delete_me'),
        ]

    if conf["ALLOW_CREATE_USER_ENDPOINT"]:
        urlpatterns += [path(f'{prefix}users/', api_views.UserView.as_view(), name="users")]

    if conf['ALLOW_MY_INFOS_ENDPOINT']:
        urlpatterns += [path(f'{prefix}users/my_infos/', api_views.users_my_infos, name="users_my_infos")]

    if conf["ALLOW_GET_ALL_USERS"]:
        urlpatterns += [path(f'{prefix}users/all/', api_views.get_users_all, name="get_users_all")]

# Classic routes

if conf["USE_CLASSIC"]:
    urlpatterns += [
        path(f'{prefix}classic_login/', classics.login_view, name='classic_login'),
        path(f'{prefix}classic_logout/', classics.logout_view, name='classic_logout'),
        path(f'{prefix}classic_signup/', classics.signup_view, name='classic_signup'),
        path(f'{prefix}classic_change_email/', classics.change_email, name='classic_change_email'),
        path(f'{prefix}classic_confirm_email/<token>', classics.confirm_email, name='classic_confirm_email'),
        path(
            f'{prefix}classic_request_resend_confirmation_email/',
            classics.request_resend_account_confirmation_email,
            name='classic_request_resend_confirmation_email'
        ),
        path(
            f'{prefix}classic_reset_password/',
            classics.reset_password,
            name='classic_reset_password_authenticated'
        ),
    ]
    if conf["ALLOW_CHANGE_USERNAME"]:
        urlpatterns += [
            path(
                f'{prefix}classic_change_username/',
                classics.change_username,
                name='classic_change_username'
            ),
        ]
    if conf["USE_CLASSIC_INDEX"]:
        urlpatterns += [path('', classics.index, name='classic_index'), ]
    if conf["USE_CLASSIC_ACCOUNT"]:
        urlpatterns += [path(f'{prefix}classic_account/', classics.account, name='classic_account'), ]

# Email Confirmation routes

if conf['CONFIRMATION_METHOD'] == 'GET':  # uses the classic views, not the 'good' way for a SPA, but works.
    urlpatterns += [
        path(f'{prefix}classic_reset_password/<token>', classics.reset_password, name='classic_reset_password'),
        path(f'{prefix}classic_request_password_reset/', classics.request_password_reset, name='classic_request_password_reset'),
    ]
    if conf["USE_SILLY"]:
        urlpatterns += [
            path(
                f'{prefix}dsa_confirm_email/<token>',
                silly_views.dsa_confirm_email,
                name='dsa_confirm_email'),
            path(
                f'{prefix}dsa_password_reset_done',
                silly_views.dsa_password_reset_done,
                name='dsa_password_reset_done'),
        ]

if conf['CONFIRMATION_METHOD'] == 'POST' or conf['AUTO_SET'] == 'TEST':
    urlpatterns += [
        path(f'{prefix}login_with_jwt/', api_views.LoginWithJWTToken.as_view(), name="login_with_jwt"),
    ]

# trying routes
if conf["TRY_TEMPLATES"] or conf['AUTO_SET'] == 'TEST':
    urlpatterns += [
        path(f'{prefix}_try/', try_views.try_templates_view, name="try_templates_view"),
        path(f'{prefix}_try_users/', try_views.try_users_view, name="try_users_view"),
        ]



from django.test import TestCase, Client
from django.urls import reverse
from django.contrib.auth import get_user_model
from django.test.client import RequestFactory


User = get_user_model()


class TestViews(TestCase):

    def setUp(self):
        self.client = Client()
        self.user = User.objects.create(
            username='testuser', email='test@test.com')
        self.user.set_password('testpass1')
        self.user.is_confirmed = True
        self.user.is_active = True
        self.user.save()

    def authenticate_user(self):
        self.client.login(username='testuser', password='testpass1')

    def test_index_view(self):
        response = self.client.get(reverse('classic_index'))
        self.assertEqual(response.status_code, 200)

    def test_login_view_GET(self):
        response = self.client.get(reverse('classic_login'))
        self.assertEqual(response.status_code, 200)

    def test_login_view_POST(self):
        response = self.client.post(reverse('classic_login'), {'credential': 'testuser', 'password': 'testpass1'})
        self.assertEqual(response.status_code, 302)

    def test_logout_view(self):
        response = self.client.get(reverse('classic_logout'))
        self.assertEqual(response.status_code, 302)

    def test_signup_view_GET(self):
        response = self.client.get(reverse('classic_signup'))
        self.assertEqual(response.status_code, 200)

    def test_signup_view_POST(self):
        response = self.client.post(
            reverse('classic_signup'),
            {
                'username': 'test_user_new',
                'email': 'new_user@mail.com',
                'password': 'testpass1',
                'password2': 'testpass1'
            }
            )
        self.assertEqual(response.status_code, 302)

    def test_change_username_GET(self):
        self.authenticate_user()
        response = self.client.get(reverse('classic_change_username'))
        self.assertEqual(response.status_code, 200)

    def test_change_username_POST(self):
        self.authenticate_user()
        response = self.client.post(
            reverse('classic_change_username'), {'username': 'test_user_new_name'})
        self.assertEqual(response.status_code, 302)

    def test_change_email_GET(self):
        self.authenticate_user()
        response = self.client.get(reverse('classic_change_email'))
        self.assertEqual(response.status_code, 200)

    def test_change_email_POST(self):
        self.authenticate_user()
        response = self.client.post(
            reverse('classic_change_email'), {'email': 'new_email@mail.com'})
        self.assertEqual(response.status_code, 302)

    def test_account_view_GET(self):
        self.authenticate_user()
        response = self.client.get(reverse('classic_account'))
        self.assertEqual(response.status_code, 200)

    def test_classic_request_resend_confirmation_email_GET(self):
        response = self.client.get(
            reverse('classic_request_resend_confirmation_email'))
        self.assertEqual(response.status_code, 200)

    def test_request_reset_password_GET(self):
        response = self.client.get(reverse('classic_request_password_reset'))
        self.assertEqual(response.status_code, 200)

    def test_request_reset_password_POST(self):
        response = self.client.post(
            reverse('classic_request_password_reset'), {'credential': 'testuser'})
        self.assertEqual(response.status_code, 200)

    def test_reset_password_GET(self):
        token = self.user.get_jwt_token()
        response = self.client.get(reverse('classic_reset_password', args=[token, ]))
        self.assertEqual(response.status_code, 200)

    def test_reset_password_POST(self):
        self.authenticate_user()
        response = self.client.post(
            reverse('classic_reset_password_authenticated'), {'password': 'testpass1', 'password2': 'testpass1'})
        self.assertEqual(response.status_code, 302)



from django import forms
from django.core.exceptions import ValidationError
from django.core.validators import (
    EmailValidator,
    MinLengthValidator,
    MaxLengthValidator
    )
from django.utils.translation import gettext_lazy as _
from django.contrib.auth.password_validation import validate_password
from django.contrib.auth import get_user_model
from django_silly_auth.config import SILLY_AUTH_SETTINGS as conf

if conf["VERBOSE"]:
    print("=== DSA IMPORT django_silly_auth.forms")

User = get_user_model()


class LoginForm(forms.Form):
    credential = forms.CharField(
        label=_("Username or email"),
        max_length=64,
        widget=forms.TextInput({
            'placeholder': _('Username or email'),
        })
    )
    password = forms.CharField(
        label=_("Password"),
        max_length=64,
        widget=forms.PasswordInput(
            {'placeholder': _('Password')}
        ),
        validators=[validate_password]
    )

    def clean_login(self):
        credential = self.cleaned_data['credential']
        if "@" in credential:
            user = User.objects.filter(email=credential)
        else:
            user = User.objects.filter(username=credential)
        if not user or not user[0].is_confirmed:
            raise ValidationError(_(
                "User '%(credential)s' unknown or unconfirmed"
                ) % {"credential": credential}
                )
        return credential


class SignUpForm(forms.Form):
    username = forms.CharField(
        label=_("Username"),
        validators=[MinLengthValidator(4), MaxLengthValidator(64)],
        max_length=64,
        widget=forms.TextInput({
            'placeholder': _('Username'),
        })
    )
    email = forms.EmailField(
        label=_("Email"),
        validators=[EmailValidator()],
        widget=forms.EmailInput({
            'placeholder': _('Email'),
        }),
        )

    password = forms.CharField(
        label=_("Password"),
        max_length=64,
        validators=[validate_password],
        widget=forms.PasswordInput({
            'placeholder': _('Password'),
        }),
    )
    password2 = forms.CharField(
        label=_("Confirm password"),
        max_length=64,
        validators=[validate_password],
        widget=forms.PasswordInput({
            'placeholder': _('Password'),
        }),
    )

    def clean_password2(self):
        password = self.cleaned_data['password']
        password2 = self.cleaned_data['password2']
        if password != password2:
            raise ValidationError(_("different than password"))
        return password2

    def clean_username(self):
        username = self.cleaned_data['username']
        if "@" in username:
            raise ValidationError(_("A username cannot include the symbol '@'."))
        else:
            user = User.objects.filter(username=username)
            if user:
                raise ValidationError(_(
                    "'%(username)s' is already taken by someone else."
                    ) % {"username": username}
                    )
        return username

    def clean_email(self):
        email = self.cleaned_data['email']
        user = User.objects.filter(email=email)
        if user:
            raise ValidationError(_(
                "'%(email)s' is already taken by someone else."
                ) % {"email": email}
                )
        return email


class CredentialForm(forms.Form):
    credential = forms.CharField(
        label=_("Username or email"),
        validators=[],
        widget=forms.TextInput({
            'placeholder': _('Username or email'),
        }),
        )

    def clean_credential(self):
        credential = self.cleaned_data['credential']
        if "@" in credential:
            user = User.objects.filter(email=credential)
        else:
            user = User.objects.filter(username=credential)
        if not user:
            raise ValidationError(_(
                "'%(credential)s' unknown or unconfirmed"
                ) % {"credential": credential}
                )
        return credential


class ResetPasswordForm(forms.Form):
    password = forms.CharField(
        label=_("Password"),
        max_length=64,
        widget=forms.PasswordInput({
            'placeholder': _('Password'),
        }),
        validators=[validate_password]
    )
    password2 = forms.CharField(
        label=_("Confirm password"),
        max_length=64,
        widget=forms.PasswordInput({
            'placeholder': _('Password'),
        }),
        validators=[validate_password]
    )

    def clean_password(self):
        password = self.cleaned_data['password']
        return password

    def clean_password2(self):
        password = self.cleaned_data['password']
        password2 = self.cleaned_data['password2']
        if password != password2:
            raise ValidationError(_("The passwords you entered do not match."))
        return password2


class ChangeUsernameForm(forms.Form):
    username = forms.CharField(
        label=_("New username"),
        validators=[MinLengthValidator(4), MaxLengthValidator(64)],
        max_length=64,
        widget=forms.TextInput({
            'placeholder': _('Username'),
        })
    )

    def clean_username(self):
        username = self.cleaned_data['username']
        if "@" in username:
            raise ValidationError(_("A username cannot include the symbol '@'."))
        else:
            user = User.objects.filter(username=username)
            if user:
                raise ValidationError(_(
                    "'%(username)s' is already taken by someone else."
                    ) % {"username": username}
                    )
        return username


class ChangeEmailForm(forms.Form):
    email = forms.EmailField(
        label=_("New email address"),
        validators=[EmailValidator()],
        widget=forms.EmailInput({
            'placeholder': _('Email'),
        }),
        )

    def clean_email(self):
        email = self.cleaned_data['email']
        user = User.objects.filter(email=email)
        if user:
            raise ValidationError(_(
                "'%(email)s' already taken by someone else."
                ) % {"email": email}
                )
        return email



from django.shortcuts import render
from django.contrib.auth import get_user_model
from django.db import transaction

from django_silly_auth.config import SILLY_AUTH_SETTINGS as conf

if conf["VERBOSE"]:
    print("=== DSA IMPORT django_silly_auth.views.views")

"""Here are the views that are non-api but can be used alongside an api site, with
 SILLY_AUTH['CONFIRMATION_METHOD'] = 'GET'

"""


User = get_user_model()


@transaction.atomic
def try_templates_view(request):
    users = User.objects.all()
    context = {
        "users": users,
        "title": "dsa render test",
        "base_template": conf["BASE_TEMPLATE"],
    }
    return render(request, "silly_auth/_try/_single_page.html", context)


@transaction.atomic
def try_users_view(request):
    users = User.objects.all()
    context = {
        "users": users,
        "title": "dsa render test",
        "base_template": conf["BASE_TEMPLATE"],
    }
    return render(request, "silly_auth/_try/_users.html", context)



from django.contrib.auth import get_user_model
from django.core import exceptions
from django.contrib.auth.password_validation import validate_password
from django.utils.translation import gettext_lazy as _

from rest_framework import serializers
from django_silly_auth.config import SILLY_AUTH_SETTINGS as conf

if conf["VERBOSE"]:
    print("=== DSA IMPORT django_silly_auth.serializers")

User = get_user_model()


class UserInfosSerializer(serializers.ModelSerializer):
    class Meta:
        model = User
        exclude = [*conf["USER_INFOS_EXCLUDE"]]


class GetAllUsersSerializer(serializers.ModelSerializer):
    class Meta:
        model = User
        fields = '__all__'


class CreateUserSerializer(serializers.ModelSerializer):
    class Meta:
        model = User
        fields = ('id', 'username', 'email', 'password')

    def validate(self, data):
        username = data.get('username')
        # email = data.get('email')
        password = data.get('password')
        username_errors = list()
        email_errors = list()
        password_errors = list()

        errors = dict()
        try:
            validate_password(
                password=password,
                user=None,
                password_validators=None)
        except exceptions.ValidationError as e:
            password_errors += list(e.messages)

        if "@" in username:
            username_errors += [_("A username cannot include the symbol '@'."), ]

        if username_errors:
            errors["username"] = username_errors
        if password_errors:
            errors["password"] = password_errors
        if email_errors:
            errors["email"] = email_errors

        if errors:
            raise serializers.ValidationError(errors)
        return data


class PasswordsSerializer(serializers.Serializer):
    password = serializers.CharField(
        write_only=True
    )
    password2 = serializers.CharField(
        write_only=True
    )

    def validate(self, data):
        password = data.get('password')
        password2 = data.get('password2')
        errors = dict()
        password_errors = list()

        if password != password2:
            password_errors += [_("The passwords you entered do not match."), ]

        try:
            validate_password(
                password=password,
                user=None,
                password_validators=None)

        except exceptions.ValidationError as e:
            password_errors += list(e.messages)

        if password_errors:
            errors['password'] = password_errors
            raise serializers.ValidationError(errors)
        return data


class EmailSerializer(serializers.Serializer):
    email = serializers.EmailField()

    def validate(self, data):
        email = data.get('email')
        errors = dict()
        email_errors = list()

        if User.objects.filter(email=email).exists():
            email_errors += [_("This email is already associated with an existing account."), ]

        if email_errors:
            errors['email'] = email_errors
            raise serializers.ValidationError(errors)
        return data


class LoginSerializer(serializers.Serializer):
    credential = serializers.CharField()
    password = serializers.CharField()

    def validate(self, data):
        credential = data.get('credential')
        password = data.get('password')
        errors = dict()
        credential_errors = list()
        password_errors = list()

        if "@" in credential:
            if not User.objects.filter(email=credential).exists():
                credential_errors += [_("Email not found"), ]
        else:
            if not User.objects.filter(username=credential).exists():
                credential_errors += [_("User not found"), ]

        if credential_errors:
            errors['credential'] = credential_errors
        if password_errors:
            errors['password'] = password_errors

        if errors:
            raise serializers.ValidationError(errors)
        return data


class CredentialJWTokenSerializer(serializers.Serializer):
    credential = serializers.CharField()
    jwt_token = serializers.CharField()

    def validate(self, data):
        credential = data.get('credential')
        jwt_token = data.get('jwt_token')
        errors = dict()
        credential_errors = list()
        jwt_errors = list()
        user = None
        if "@" in credential:
            if not User.objects.filter(email=credential).exists():
                credential_errors += [_("Email not found"), ]
            else:
                user = User.objects.filter(email=credential).first()
        else:
            if not User.objects.filter(username=credential).exists():
                credential_errors += [_("User not found"), ]
            else:
                user = User.objects.filter(username=credential).first()
        if credential_errors:
            errors['credential'] = credential_errors

        jwt_user = User.verify_jwt_token(jwt_token)
        if (user is not None and jwt_user is not None) and user != jwt_user:
            jwt_errors += [_("The given token does not match the user")]
        if User.verify_jwt_token(jwt_token) is None:
            jwt_errors += [_("jwt token invalid or expired")]
        if jwt_errors:
            errors['jwt_token'] = jwt_errors
        if errors:
            raise serializers.ValidationError(errors)
        return data


class UsernameSerializer(serializers.Serializer):
    username = serializers.CharField()

    def validate(self, data):
        username = data.get('username')
        errors = dict()
        username_errors = list()

        if User.objects.filter(username=username).exists():
            username_errors += [_("This username is already associated with an existing account."), ]

        if "@" in username:
            username_errors += [_("A username cannot include the symbol '@'."), ]

        if username_errors:
            errors['username'] = username_errors
            raise serializers.ValidationError(errors)
        return data




from django.contrib.auth import get_user_model
from django.shortcuts import render
from django.utils.translation import gettext_lazy as _
from django.db import transaction

from django_silly_auth.config import SILLY_AUTH_SETTINGS as conf


if conf["VERBOSE"]:
    print("=== DSA IMPORT django_silly_auth.views.classics")

User = get_user_model()


@transaction.atomic
def dsa_confirm_email(request, token):
    user = User.verify_jwt_token(token)
    if user is None:
        msg = _("Token invalid or expired"),
        tag = "danger"
    if user is not None and user.is_active:
        if not user.is_confirmed:
            user.is_confirmed = True
            user.new_email = None
            msg = _("Your account has been confirmed")
            tag = "success"
            user.save()
        elif user.new_email:
            user.email = user.new_email
            user.new_email = None
            msg = _("Your new email has been confirmed")
            tag = "success"
            user.save()
        else:
            msg = _("Your account has already been confirmed.")
            tag = "warning"

    context = {
        "base_template": conf["BASE_TEMPLATE"],
        "message": msg,
        "tag": tag,
        "link": conf["SILLY_LINK_TO_SPA"],
        "site_name": conf["SITE_NAME"],
    }
    return render(request, 'silly_auth/silly/silly_confirm_email.html', context)


def dsa_password_reset_done(request):
    msg = _("Your password has been successfully reset.")
    context = {
        "base_template": conf["BASE_TEMPLATE"],
        "message": msg,
        "tag": "success",
        "link": conf["SILLY_LINK_TO_SPA"],
        "site_name": conf["SITE_NAME"],
    }
    return render(request, 'silly_auth/silly/silly_confirm_email.html', context)



from django.contrib.auth import get_user_model
from django.urls import reverse

from rest_framework .test import APITestCase
from rest_framework.authtoken.models import Token

User = get_user_model()


class TestApiViews(APITestCase):

    def setUp(self):
        self.user = User.objects.create_user(
            username='testuser', email='test@test.com', password='testpass1')
        self.user.is_confirmed = True
        self.user.is_active = True
        self.user.save()
        self.token = Token.objects.create(user=self.user)

    def api_authentication(self):
        self.client.credentials(HTTP_AUTHORIZATION='Token ' + self.token.key)

    def new_user(self):
        user = User.objects.create_user(
            username='new_user', email='new_user@mail.com', password='testpass1')
        user.is_confirmed = True
        user.is_active = True
        user.save()
        return user

    def auth_token(self, user):
        return Token.objects.create(user=user)

    def api_authenticate(self, user):
        self.client.credentials(HTTP_AUTHORIZATION='Token ' + self.auth_token(user).key)

    def test_login(self):
        url = reverse('token_login')
        response = self.client.post(
            url, {'credential': 'testuser', 'password': 'testpass1'})
        self.assertEqual(response.status_code, 200)

    def test_logout(self):
        self.api_authentication()
        url = reverse('token_logout')
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)

    def test_email_confirm_email_resend(self):
        self.user.is_confirmed = False
        self.user.save()
        url = reverse('email_confirm_email_resend')
        response = self.client.post(url, {'credential': 'testuserx'})
        self.assertEqual(response.status_code, 400)
        response = self.client.post(url, {'credential': 'testuser'})
        self.assertEqual(response.status_code, 200)

    def test_password_request_reset(self):
        url = reverse('password_request_reset')
        response = self.client.post(url, {'credential': 'testuserx'})
        self.assertEqual(response.status_code, 400)
        response = self.client.post(url, {'credential': 'testuser'})
        self.assertEqual(response.status_code, 200)

    def test_password_change(self):
        self.api_authentication()
        url = reverse('password_change')
        response = self.client.post(url, {'password': 'testpass1', 'password2': 'testpass1x'})
        self.assertEqual(response.status_code, 400)
        response = self.client.post(url, {'password': 'testpass1', 'password2': 'testpass1'})
        self.assertEqual(response.status_code, 200)

    def test_email_request_change(self):
        self.api_authentication()
        url = reverse('email_request_change')
        response = self.client.post(url, {'email': 'new_email@mail.com'})
        self.assertEqual(response.status_code, 200)
        response = self.client.post(url, {'email': 'new_email_mail.com'})
        self.assertEqual(response.status_code, 400)

    def test_username_change(self):
        self.api_authentication()
        url = reverse('username_change')
        response = self.client.post(url, {'username': 'new_username'})
        self.assertEqual(response.status_code, 200)

    def test_users_my_infos(self):
        self.api_authentication()
        url = reverse('users_my_infos')
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)

    def test_users_delete_me(self):
        self.assertEqual(User.objects.count(), 1)
        self.api_authentication()
        url = reverse('users_delete_me')
        response = self.client.delete(url)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(User.objects.count(), 0)



from functools import wraps
from threading import Thread
import time

from django.conf import settings
from django.shortcuts import reverse
from django.core.mail import send_mail
from django.template.loader import get_template
from django.utils.translation import gettext_lazy as _

from smtplib import SMTPServerDisconnected

from django_silly_auth.config import SILLY_AUTH_SETTINGS as conf


if conf["VERBOSE"]:
    print("=== DSA IMPORT django_silly_auth.utils")


def dsa_thread(func):
    """decorator that simply runs the function in parallel thread"""
    @wraps(func)
    def wrapper(*args, **kwargs):
        send = Thread(target=func, args=args, kwargs=kwargs, daemon=True)
        send.start()
    return wrapper


@dsa_thread
def delete_unconfirmed(user, *args, **kwargs):
    if conf["VERBOSE"]:
        print(
            f"=== {user} created, waiting for confirmation..."
            f"{conf['DELETE_UNCONFIRMED_TIME']} hours")
    if conf["DELETE_UNCONFIRMED_TIME"] > 0.0:
        time.sleep(conf["DELETE_UNCONFIRMED_TIME"] * 3600.0)
        if not user.is_confirmed:
            if conf["VERBOSE"]:
                print(f"=== {user} deleted")
            user.delete()


def dsa_send_mail(*args, **kwargs):
    send = Thread(target=send_mail, args=args, kwargs=kwargs, daemon=True)
    send.start()


def send_password_reset_email(request, user):
    token = user.get_jwt_token(expires_in=conf["EMAIL_VALID_TIME"])
    domain = request.build_absolute_uri('/')[:-1]
    if conf["CONFIRMATION_METHOD"] == 'GET':
        link = domain + reverse('classic_reset_password', args=[token])
    if conf["CONFIRMATION_METHOD"] == 'POST':
        link = conf['SPA_EMAIL_LOGIN_LINK'] + f"{token}"
    context = {
        'user': user,
        'link': link,
        'site_name': conf["SITE_NAME"]
    }

    msg_text = get_template(conf["EMAIL_RESET_PASSWORD_TEMPLATE"])
    if conf["EMAIL_TERMINAL_PRINT"]:
        print("from ", settings.EMAIL_HOST_USER)
        print(msg_text.render(context))

    dsa_send_mail(
        'Password reset request',
        msg_text.render(context),
        settings.EMAIL_HOST_USER,
        [user.email],
        fail_silently=False,
    )


def send_confirm_email(request, user, new_email=False):
    token = user.get_jwt_token(expires_in=conf["EMAIL_VALID_TIME"])
    domain = request.build_absolute_uri('/')[:-1]
    # if new_email:
    if conf["USE_SILLY"]:
        link = domain + reverse('dsa_confirm_email', args=[token])
    elif conf["CONFIRMATION_METHOD"] == 'GET':
        link = domain + reverse('classic_confirm_email', args=[token])
    if conf["CONFIRMATION_METHOD"] == 'POST':
        link = conf['SPA_EMAIL_LOGIN_LINK'] + f"{token}"
    context = {
        'user': user,
        'link': link,
        'site_name': conf["SITE_NAME"],
    }

    msg_text = get_template(conf["EMAIL_CONFIRM_ACCOUNT_TEMPLATE"])

    if conf["EMAIL_TERMINAL_PRINT"]:
        print("from ", settings.EMAIL_HOST_USER)
        print(msg_text.render(context))

    if new_email:
        email = user.new_email
    else:
        email = user.email

    dsa_send_mail(
        'Confirm your new email',
        msg_text.render(context),
        settings.EMAIL_HOST_USER,
        [email],
        fail_silently=False,
    )



from django.contrib.auth import get_user_model
from django.utils.translation import gettext_lazy as _
from django.db import transaction
from rest_framework.response import Response
from rest_framework.views import APIView
from rest_framework.decorators import api_view, permission_classes
from rest_framework.permissions import AllowAny, IsAuthenticated
from rest_framework.exceptions import (
    ValidationError,
    )

from django_silly_auth.serializers import (
    UserInfosSerializer,
    GetAllUsersSerializer,
    CreateUserSerializer,
    PasswordsSerializer,
    EmailSerializer,
    UsernameSerializer,
    )
from django_silly_auth.config import SILLY_AUTH_SETTINGS as conf
from django_silly_auth.utils import (
    send_password_reset_email,
    send_confirm_email,
    delete_unconfirmed
)


if conf["VERBOSE"]:
    print("=== DSA IMPORT django_silly_auth.views.api_view_if_drf")

User = get_user_model()


@transaction.atomic
@api_view(['POST'])
@permission_classes([AllowAny])
def email_confirm_email_resend(request):
    """Resends an email to the user to confirm his account"""
    credential = request.data.get('credential')
    if not credential:
        msg = _("no credential provided")
        raise ValidationError({"detail": msg}, code='authorization')
    if "@" in credential:
        user = User.objects.filter(email=credential).first()
    else:
        user = User.objects.filter(username=credential).first()
    if user:
        if user.is_confirmed:
            msg = _("Your account is already confirmed.")
            raise ValidationError({"detail": msg}, code='authorization')
        send_confirm_email(request, user)
        return Response({'success': _('Email sent for password reset')})
    msg = "Invalid credential"
    raise ValidationError({"detail": msg}, code='authorization')


@transaction.atomic
@api_view(['GET'])
@permission_classes([IsAuthenticated])
def token_logout(request):
    """Destroys the auth token"""
    request.user.auth_token.delete()
    return Response({'success': _('Logged out.')})


@transaction.atomic
@api_view(['POST'])
@permission_classes([AllowAny])
def password_request_reset(request):
    """Sends an email to the user with a link to reset their password"""
    credential = request.data.get('credential')
    if not credential:
        msg = _("No credentials were provided")
        raise ValidationError({"detail": msg}, code='authorization')
    if "@" in credential:
        user = User.objects.filter(email=credential).first()
    else:
        user = User.objects.filter(username=credential).first()
    if user:
        send_password_reset_email(request, user)
        return Response({'success': _("Email sent for password reset")})
    msg = _("Invalid credential")
    raise ValidationError({"detail": msg}, code='authorization')


class UserView(APIView):
    permission_classes = []

    @transaction.atomic
    def post(self, request, format=None):
        """Create a new user"""
        serializer = CreateUserSerializer(data=request.data)
        if serializer.is_valid():
            user = serializer.save()
            user.set_password(request.data['password'])
            user.save()
            delete_unconfirmed(user)
            message1 = _(
                "Please check your inbox at '%(email)s' to confirm your account. "
                ) % {'email': user.email}
            if conf["DELETE_UNCONFIRMED_TIME"] != 0:
                message2 = _(
                    "If you do not confirm your account within the next "
                    "%(hours)s hours, it will be deleted."
                    ) % {'hours': conf['DELETE_UNCONFIRMED_TIME']}

                message = "%(message1)s %(message2)s" % {
                    'message1': message1,
                    'message2': message2,
                }
            serializer = UserInfosSerializer(user)
            msg = {
                "user": serializer.data,
                "message": message,
            }

            send_confirm_email(request, user)

            return Response(msg)
        else:
            error = serializer.errors
            raise ValidationError(error, code='authorization')


@api_view(['GET'])
def get_users_all(request):
    """!! FOR DEV ONLY !! Get all users"""
    users = User.objects.all()
    serializer = GetAllUsersSerializer(users, many=True)
    return Response(serializer.data)

@transaction.atomic
@api_view(['POST'])
@permission_classes([IsAuthenticated])
def password_change(request):
    """Changes the user's password"""
    serializer = PasswordsSerializer(data=request.data)
    if serializer.is_valid():
        user = request.user
        password = request.data.get('password')
        user.set_password(password)
        user.save()
        return Response({'success': _('Password successfully changed.')})
    error = serializer.errors
    raise ValidationError(error, code='authorization')


@transaction.atomic
@api_view(['POST'])
@permission_classes([IsAuthenticated])
def email_request_change(request):
    user = request.user
    serializer = EmailSerializer(data=request.data)
    if serializer.is_valid():
        new_email = request.data.get('email')
        user.new_email = new_email
        user.save()
        send_confirm_email(request, user, new_email=True)

        return Response(
            {'success': _(
                "New email saved, check your inbox at '%(new_email)s' "
                "to activate it."
            ) % {'new_email': new_email}}
        )
    error = serializer.errors
    raise ValidationError(error, code='authorization')



@transaction.atomic
@api_view(['POST'])
@permission_classes([IsAuthenticated])
def username_change(request):
    """Changes the user's username"""
    serializer = UsernameSerializer(data=request.data)
    if serializer.is_valid():
        user = request.user
        username = request.data.get('username')
        user.username = username
        user.save()
        return Response({'success': _('Username successfully changed.')})
    error = serializer.errors
    raise ValidationError(error, code='authorization')


@api_view(['GET'])
@permission_classes([IsAuthenticated])
def users_my_infos(request):
    """Returns the user's infos"""
    serializer = UserInfosSerializer(request.user)
    return Response(serializer.data)

@api_view(['DELETE'])
@permission_classes([IsAuthenticated])
def users_delete_me(request):
    """Deletes the user's account"""
    request.user.auth_token.delete()
    request.user.delete()
    return Response({'success': _('Account successfully deleted.')})



from django.test import TestCase, Client
from django.urls import reverse
from django.contrib.auth import get_user_model
from django.contrib.auth import login, logout, authenticate
from django.test.client import RequestFactory


User = get_user_model()


class TestViews(TestCase):

    def setUp(self):
        self.client = Client()
        self.user = User.objects.create(
            username='testuser', email='test@test.com')
        self.user.set_password('testpass1')
        self.user.is_confirmed = True
        self.user.is_active = True
        self.user.save()

    def authenticate_user(self):
        self.client.login(username='testuser', password='testpass1')

    def test_dsa_confirm_email(self):
        token = self.user.get_jwt_token()
        response = self.client.get(reverse('dsa_confirm_email', args=[token]))
        self.assertEqual(response.status_code, 200)

    def dsa_password_reset_done(self):
        response = self.client.get(reverse('dsa_password_reset_done'))
        self.assertEqual(response.status_code, 200)



from django.conf import settings


class SillyAuthError(Exception):
    pass


SILLY_AUTH_SETTINGS = {
    # Quick settings
    "AUTO_SET": 'TRY',  # 'CLASSIC', 'SPA', 'TRY', 'SILLY' or 'TEST'
    "DSA_PREFIX": 'auth/',  # only the CLASSIC_INDEX view has a blank prefix, it's the entry point in a classic site.

    # Secondary settings
    "SITE_NAME": None,  # str used in templates if provided
    "DELETE_UNCONFIRMED_TIME": 24.0,  # hours after what an unconfirmed account is deleted, O to set off
    "ALLOW_CHANGE_USERNAME": True,  # allows the change username endpoint


    # Classic settings
    "USE_CLASSIC": True,
        # redirections
    "USE_CLASSIC_INDEX": True,  # if False, your index route must have the name='classic_index'
    "USE_CLASSIC_ACCOUNT": True,  # if False, your account route must have the name='classic_account'
        # templates settings
    "TEMPLATES_TITLE": "D.S. AUTH",  # title if you use the provided templates
    "BASE_TEMPLATE": "silly_auth/_base.html",  # if you use the provided templates
        # templates paths
    "CLASSIC_INDEX": "silly_auth/classic/index.html",
    "CLASSIC_ACCOUNT": "silly_auth/classic/account.html",
    "CLASSIC_SIGNUP": "silly_auth/classic/signup.html",
    "CLASSIC_LOGIN": "silly_auth/classic/login.html",
    "CLASSIC_CHANGE_EMAIL": "silly_auth/classic/change_email.html",
    "CLASSIC_CHANGE_USERNAME": "silly_auth/classic/change_username.html",
    "CLASSIC_REQUEST_PASSWORD_RESET": "silly_auth/classic/request_password_reset.html",
    "CLASSIC_RESET_PASSWORD": "silly_auth/classic/reset_password.html",
    "CLASSIC_REQUEST_RESEND_ACCOUNT_CONFIRMATION_EMAIL": "silly_auth/classic/request_resend_account_confirmation_email.html",

    # DRF settings
    "USE_DRF": False,  # False for only classic django views
    "CONFIRMATION_METHOD": 'GET',  # 'GET' or 'POST'
    "ALLOW_CREATE_USER_ENDPOINT": True,  # activate this endpoint
    "ALLOW_MY_INFOS_ENDPOINT": True,  # activate this endpoint
    "ALLOW_DELETE_ME_ENDPOINT": True,  # activate this endpoint
    "USER_INFOS_EXCLUDE": ['password', 'id'],  # fields to exclude from the user infos

    # pure SPA only:
    "SPA_EMAIL_LOGIN_LINK": "http://your spa address/",  # + <jwt_token>",


    # Silly settings
    "USE_SILLY": False,
    "SILLY_LINK_TO_SPA": None,  # link used in templates to get back to your SPA

    # emails settings
    "EMAIL_TERMINAL_PRINT": True,  # print emails to terminal
    "EMAIL_VALID_TIME": 1200,  # seconds
    "EMAIL_CONFIRM_ACCOUNT_TEMPLATE":
        "silly_auth/emails/confirm_email.txt",
    "EMAIL_RESET_PASSWORD_TEMPLATE":  # email template
        "silly_auth/emails/request_password_reset.txt",

    # For development,
    "TRY_TEMPLATES": False,  # for dev only,  opens 2 "_test/" endpoint
    "VERBOSE": False,  # prints to terminal : imports
    "ALLOW_GET_ALL_USERS": False,  # for DEV ONLY,  opens 'users/all/' endpoint
}


# Overwrite SILLY_AUTH_SETTINGS with datas from settings.SILLY_AUTH

try:
    settings.SILLY_AUTH
    is_set = True
except AttributeError:
    is_set = False

if is_set and "AUTO_SET" in settings.SILLY_AUTH:
    auto_set = settings.SILLY_AUTH['AUTO_SET']
    if auto_set not in ['CLASSIC', 'SPA', 'SILLY', 'TRY', 'TEST']:
        raise SillyAuthError(
            "AUTO_SET must be 'CLASSIC', 'SPA', 'SILLY', 'TRY' or 'TEST'")
else:
    auto_set = SILLY_AUTH_SETTINGS["AUTO_SET"]

match auto_set:
    case "CLASSIC":
        SILLY_AUTH_SETTINGS["USE_DRF"] = False
        SILLY_AUTH_SETTINGS["USE_CLASSIC"] = True
        SILLY_AUTH_SETTINGS["CONFIRMATION_METHOD"] = 'GET'

    case "SPA":
        SILLY_AUTH_SETTINGS["USE_DRF"] = True
        SILLY_AUTH_SETTINGS["USE_CLASSIC"] = False
        SILLY_AUTH_SETTINGS["CONFIRMATION_METHOD"] = 'POST'

    case "SILLY":
        SILLY_AUTH_SETTINGS["USE_DRF"] = True
        SILLY_AUTH_SETTINGS["USE_CLASSIC"] = False
        SILLY_AUTH_SETTINGS["CONFIRMATION_METHOD"] = 'GET'
        SILLY_AUTH_SETTINGS["USE_SILLY"] = True

    case "TRY":
        SILLY_AUTH_SETTINGS["USE_DRF"] = False
        SILLY_AUTH_SETTINGS["USE_CLASSIC"] = True
        SILLY_AUTH_SETTINGS["CONFIRMATION_METHOD"] = 'GET'
        SILLY_AUTH_SETTINGS["BASE_TEMPLATE"] = "silly_auth/_try/_base.html"
        SILLY_AUTH_SETTINGS["TRY_TEMPLATES"] = True
        SILLY_AUTH_SETTINGS["VERBOSE"] = True

    case "TEST":
        SILLY_AUTH_SETTINGS["USE_DRF"] = True
        SILLY_AUTH_SETTINGS["ALLOW_CREATE_USER_ENDPOINT"] = True,
        SILLY_AUTH_SETTINGS["ALLOW_MY_INFOS_ENDPOINT"] = True,
        SILLY_AUTH_SETTINGS["ALLOW_DELETE_ME_ENDPOINT"] = True,
        SILLY_AUTH_SETTINGS["USE_CLASSIC"] = True
        SILLY_AUTH_SETTINGS["USE_CLASSIC_INDEX"] = True,
        SILLY_AUTH_SETTINGS["USE_CLASSIC_ACCOUNT"] = True,
        SILLY_AUTH_SETTINGS["USE_SILLY"] = True
        SILLY_AUTH_SETTINGS["CONFIRMATION_METHOD"] = 'GET'
        SILLY_AUTH_SETTINGS["BASE_TEMPLATE"] = "silly_auth/_try/_base.html"
        SILLY_AUTH_SETTINGS["EMAIL_TERMINAL_PRINT"] = False

if is_set:
    for key in settings.SILLY_AUTH:
        if key not in SILLY_AUTH_SETTINGS:
            raise SillyAuthError(f"Unexpected key in settings.SILLY_AUTH: '{key}'")
        SILLY_AUTH_SETTINGS[key] = settings.SILLY_AUTH[key]


if SILLY_AUTH_SETTINGS["CONFIRMATION_METHOD"] == 'POST' and "SPA_EMAIL_LOGIN_LINK" not in settings.SILLY_AUTH:
    raise SillyAuthError("Confirmation method is 'POST', you must define a SPA_EMAIL_LOGIN_LINK")

try:
    float(SILLY_AUTH_SETTINGS["DELETE_UNCONFIRMED_TIME"])
    assert SILLY_AUTH_SETTINGS["DELETE_UNCONFIRMED_TIME"] >= 0
except (ValueError, AssertionError):
    raise SillyAuthError("DELETE_UNCONFIRMED_TIME must be a positive float, or 0 to set off")


if SILLY_AUTH_SETTINGS["VERBOSE"]:
    print("=== DSA IMPORT django_silly_auth.config")



#! /usr/bin/env python3
# coding: utf-8

"""
REMINDER:
1- build
./setup.py sdist bdist_wheel
2- basic verifications
twine check dist/*
2.5- Deploy on testpypi (optionnal, site here : https://test.pypi.org/):
twine upload --repository testpypi dist/*
3- upload to PyPi
twine upload dist/*
"""

from django_silly_auth import __version__
import pathlib
from setuptools import setup


# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()

# This call to setup() does all the work
setup(
    name="django-silly-auth",
    version=f"{__version__}",
    description=(
        "Authentication package for Django and DRF"
        ),
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/byoso/django_silly_auth",
    author="Vincent Fabre",
    author_email="peigne.plume@gmail.com",
    license="MIT",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Development Status :: 4 - Beta",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
    ],
    packages=[
        "django_silly_auth",
        "django_silly_auth.templates",
        "django_silly_auth.templates.silly_auth",
        "django_silly_auth.templates.silly_auth._try",
        "django_silly_auth.templates.silly_auth.classic",
        "django_silly_auth.templates.silly_auth.emails",
        "django_silly_auth.templates.silly_auth.silly",
        "django_silly_auth.views",
        "django_silly_auth.locale",
        "django_silly_auth.locale.fr",
        "django_silly_auth.locale.fr.LC_MESSAGES",
        "django_silly_auth.locale.en",
        "django_silly_auth.locale.en.LC_MESSAGES",
        "django_silly_auth.tests",
        ],
    # include_package_data=True,
    package_data={'': ['*.txt', '*.html', '*.po', '*.mo', '*.pot']},
    python_requires='>=3.7',
    install_requires=[
        "pyjwt >= 2.6.0",
    ],
    keywords='django auth drf jwt',
    # entry_points={
    #     "console_scripts": [
    #         "django_silly_auth=main.cmd:cmd",
    #     ]
    # },
    setup_requires=['wheel'],
)



from django_silly_auth.config import SILLY_AUTH_SETTINGS as conf

if conf["VERBOSE"]:
    print("=== DSA IMPORT django_silly_auth.views.api_views")
    if conf["USE_DRF"]:
        print("=== DSA IMPORT django_silly_auth.views.api_views_if_drf")

if conf["USE_DRF"]:
    from django_silly_auth.views.api_views_if_drf import *
    from django_silly_auth.views.api_custom_login import (
        LoginWithAuthToken,
        LoginWithJWTToken,
    )




from django.db import transaction
from django.utils.translation import gettext_lazy as _
from django.contrib.auth import get_user_model


from rest_framework.views import APIView
from rest_framework.authtoken.models import Token
from rest_framework.response import Response
from rest_framework.authtoken.views import ObtainAuthToken
from rest_framework.serializers import ValidationError
from rest_framework.decorators import api_view, permission_classes
from rest_framework.permissions import AllowAny

from django_silly_auth.serializers import (
    LoginSerializer,
    CredentialJWTokenSerializer
    )
from django_silly_auth.config import SILLY_AUTH_SETTINGS as conf

if conf["VERBOSE"]:
    print("=== DSA IMPORT django_silly_auth.views.api_custom_login")

User = get_user_model()


class LoginWithAuthToken(ObtainAuthToken):

    def post(self, request, *args, **kwargs):
        """Login view modified to use email or username as credential"""
        serializer = LoginSerializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        credential = serializer.validated_data['credential']
        if "@" in credential:
            user = User.objects.filter(email=credential).first()
        else:
            user = User.objects.filter(username=credential).first()
        password = serializer.validated_data['password']
        match = False
        if user:
            match = user.check_password(password)
        if match:
            if not user.is_confirmed and not user.is_superuser:
                msg = _(
                    'Your account has not been confirmed yet. '
                    'Please check your inbox for a confirmation link.')
                raise ValidationError({'detail': msg}, code='authorization')
            token, created = Token.objects.get_or_create(user=user)
            return Response({'auth_token': token.key})
        msg = _('Incorrect credentials.')
        raise ValidationError({'detail': [msg, ]}, code='authorization')


class LoginWithJWTToken(APIView):
    @transaction.atomic
    def post(self, request, *args, **kwargs):
        serializer = CredentialJWTokenSerializer(data=request.data)
        if serializer.is_valid():
            jwt_token = serializer.validated_data['jwt_token']
            user = User.verify_jwt_token(jwt_token)
            if not user.is_confirmed:
                user.is_confirmed = True
                user.new_email = None
                user.save()
                msg = _("Your account has been confirmed.")
            elif user.new_email:
                user.email = user.new_email
                user.new_email = None
                user.save()
                msg = _("Your new email has been confirmed.")
            else:
                msg = _("You've been logged in via email confirmation, "
                        "please change your password if necessary.")
            token, created = Token.objects.get_or_create(user=user)
            return Response({'token': token.key, 'message': msg})
        raise ValidationError(serializer.errors, code='authorization')
